package denim;

import beast.core.Description;
import beast.core.Input;
import beast.core.Operator;
import beast.core.parameter.RealParameter;
import beast.util.Randomizer;

/*
 * Created by Work on 01/11/2016.
 */

@Description("An operator which changes the migration parameters in a gene tree.")
public class MigrationPointsWalker extends Operator {


    @SuppressWarnings({"CanBeFinal", "WeakerAccess"})
    public Input<RealParameter> migPointsInput =
            new Input<>("migPoints",
                    "A vectors of reals in [0,1]", Input.Validate.REQUIRED);

    @Override
    public void initAndValidate() {

    }



    @Override
    public double proposal() {
        return doWalks();

    }


    // Changes each element in a vector of values in [0,1] with probability p, where p is uniform random in [0,1].
    // The size of the change is also randomized.
    private double doWalks() {
        double logHR = 0.0;
        double halfWidth = Randomizer.uniform(0.0, 0.5);
        double p = Randomizer.nextDouble();
        RealParameter migPoints = migPointsInput.get();
        for (int i = 0; i < migPoints.getDimension(); i++) {
            if (Randomizer.nextDouble() > p) {
                double x = migPoints.getArrayValue(i);
                x += Randomizer.uniform(-halfWidth, halfWidth);
                if (x < 0) { x = -x; }
                if (x >= 1) { x = 2.0 - x; }
                migPoints.setValue(i, x);
            }
        }
        return logHR;
    }
}
